<?php
/**
 * @copyright (C) 2021 - 2023 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

// Handles the relationship with the _extensions table
class WADA_Extensions
{
    public static function getAllExtensions($activeOnly = false){
        global $wpdb;
        $query = 'SELECT * FROM '. WADA_Database::tbl_extensions();
        if($activeOnly){
            $query .= ' WHERE active > 0';
        }
        $query .= ' ORDER BY name, id';
        return $wpdb->get_results($query);
    }

    /**
     * @param $pluginFolder
     * @return object|null
     */
    public static function getExtensionByPluginFolder($pluginFolder){
        global $wpdb;
        $query = 'SELECT * FROM '. WADA_Database::tbl_extensions().' WHERE plugin_folder = %s';
        return $wpdb->get_row($wpdb->prepare($query, $pluginFolder));
    }

    /**
     * Updates the extension table according to the plugin's status in WordPress.
     * Will not create a new entry, if extension not existing.
     * Only installed and activated plugins are active in our extensions table.
     * @param string $pluginFolder
     * @return bool
     */
    public static function updateExtensionStatus($pluginFolder){
        global $wpdb;
        $extension = self::getExtensionByPluginFolder($pluginFolder);
        $extensionId = $extension ? intval($extension->id) : 0;
        $wasActiveBefore = $extension ? (intval($extension->active) > 0) : false;
        WADA_Log::debug('updateExtensionStatus pluginFolder: '.$pluginFolder.', extensionId: '.$extensionId.' (old status: '.($wasActiveBefore ? 'active':'inactive').')');

        if($extensionId){
            $isInstalledAndActive = WADA_PluginUtils::isPluginActive($pluginFolder);
            $active = $isInstalledAndActive ? 1 : 0;
            WADA_Log::debug('updateExtensionStatus pluginFolder: '.$pluginFolder.', set to new active status: '.$active);

            // if active before, and now no longer (i.e. active was set to 0)
            if($wasActiveBefore && !$isInstalledAndActive){
                WADA_Log::debug('updateExtensionStatus pluginFolder: '.$pluginFolder.' gets inactivated, deactivate sensors first');
                self::deactivateSensorsOfExtension($extensionId);
            }

            $query = 'UPDATE ' . WADA_Database::tbl_extensions();
            $query .= ' SET active = \''.$active.'\'';
            $query .= ' WHERE id=\''.$extensionId.'\'';
            $affRows = $wpdb->query( $query );

            if($affRows === false){
                return false;
            }

            return true;
        }
        return false;
    }

    public static function deactivateSensorsOfExtension($extensionId){
        global $wpdb;
        WADA_Log::debug('deactivateSensorsOfExtension of extensionId: '.$extensionId);
        $query =  'UPDATE ' . WADA_Database::tbl_sensors();
        $query .= ' SET active = \'0\'';
        $query .= ' WHERE extension_id = %d';
        $affRows = $wpdb->query($wpdb->prepare($query, $extensionId));

        if($affRows === false){
            return false;
        }

        WADA_Log::debug('deactivateSensorsOfExtension extensionId: '.$extensionId.', affRows: '.$affRows);
        return true;
    }
    
}