<?php
/**
 * @copyright (C) 2021 - 2023 Holger Brandt IT Solutions
 * @license GPL2
*/

if(!defined('ABSPATH')){
    exit;
}

class WADA_DateUtils
{
    public static function getUTCforMySQLTimestamp($dateTime=null){
        if(is_null($dateTime)){
            $dateTime = 'now';
        }
        try {
            $dateUTC = new DateTime($dateTime, new DateTimeZone('UTC'));
            return $dateUTC->format('Y-m-d H:i:s');
        } catch (Exception $e) {
            WADA_Log::error('WADA_DateUtils/getUTCforMySQLTimestamp error: '.$e->getMessage());
            WADA_Log::error('dateTime: '.$dateTime);
        }
        return false;
    }


    public static function getUTCforMySQLDate($dateTime=null){
        if(is_null($dateTime)){
            $dateTime = 'now';
        }
        try {
            $dateUTC = new DateTime($dateTime, new DateTimeZone('UTC'));
            return $dateUTC->format('Y-m-d');
        } catch (Exception $e) {
            WADA_Log::error('WADA_DateUtils/getUTCforMySQLDate error: '.$e->getMessage());
            WADA_Log::error('dateTime: '.$dateTime);
        }
        return false;
    }

    public static function getUTCforMySQLTimestampFromUnixTime($unixTimestamp){
        $unixTimestampInt = intval($unixTimestamp);
        try {
            $dateUTC = new DateTime('@' . $unixTimestampInt);
            return $dateUTC->format('Y-m-d H:i:s');
        } catch (Exception $e) {
            WADA_Log::error('WADA_DateUtils/getUTCforMySQLTimestampFromUnixTime error: '.$e->getMessage());
            WADA_Log::error('unixTimestamp: '.$unixTimestamp);
        }
        return false;
    }

    protected static function formatUTCforWp($utcDateTimeStr, $formatString, $timeZone = null){
        if(!$timeZone){
            $timeZone = wp_timezone();
        }
        try {
            $dateUTC = new DateTime($utcDateTimeStr, new DateTimeZone('UTC'));
            $dateUTC->setTimezone($timeZone);
            return $dateUTC->format($formatString);
        } catch (Exception $e) {
            WADA_Log::error('WADA_DateUtils/formatUTCforWp error: '.$e->getMessage());
            WADA_Log::error('utcDateTimeStr: '.$utcDateTimeStr);
            WADA_Log::error('formatString: '.$formatString);
            WADA_Log::error('timeZone (name): '.(($timeZone && method_exists($timeZone, 'getName')) ? $timeZone->getName() : ''));
        }
        return false;
    }

    /**
     * @param string $utcDateTimeStr
     * @param null|string $formatString
     * @param null|DateTimeZone $timeZone
     * @return false|string
     */
    public static function formatUTCasDatetimeForWP($utcDateTimeStr, $formatString = null, $timeZone = null){
        if(!$formatString){
            $formatString = self::getDefaultDateTimeFormat();
        }
        return self::formatUTCforWp($utcDateTimeStr, $formatString, $timeZone);
    }

    /**
     * @param string $utcDateTimeStr
     * @param null|string $formatString
     * @param null|DateTimeZone $timeZone
     * @return false|string
     */
    public static function formatUTCasDateForWP($utcDateTimeStr, $formatString = null, $timeZone = null){
        if(!$formatString){
            $formatString = self::getDefaultDateFormat();
        }
        return self::formatUTCforWp($utcDateTimeStr, $formatString, $timeZone);
    }

    public static function timeAgo($utcDateTimeStr){
        $from = self::formatUTCdateTimeAsUnixTimestamp($utcDateTimeStr);
        $to = self::getUTCUnixTimestamp();
        $diff = human_time_diff($from, $to);
        return sprintf(__('%s ago', 'wp-admin-audit'), $diff);
    }

    public static function getUTCUnixTimestamp(){
        return self::formatUTCdateTimeAsUnixTimestamp('now');
    }

    public static function formatUTCdateTimeAsUnixTimestamp($utcDateTimeStr){
        $dateUTC = new DateTime($utcDateTimeStr, new DateTimeZone('UTC'));
        return $dateUTC->getTimestamp();
    }

    public static function formatWPDatetimeForWP($wpDateTimeStr, $formatString = null, $timeZone = null){
        if(!$formatString){
            $formatString = self::getDefaultDateTimeFormat();
        }
        if(!$timeZone){
            $timeZone = wp_timezone();
        }
        try {
            $dateWP = new DateTime($wpDateTimeStr, $timeZone);
            return $dateWP->format($formatString);
        } catch (Exception $e) {
            WADA_Log::error('WADA_DateUtils/formatWPDatetimeForWP error: '.$e->getMessage());
            WADA_Log::error('wpDateTimeStr: '.$wpDateTimeStr);
            WADA_Log::error('formatString: '.$formatString);
            WADA_Log::error('timeZone (name): '.(($timeZone && method_exists($timeZone, 'getName')) ? $timeZone->getName() : ''));
        }
        return false;
    }

    protected static function getDefaultDateTimeFormat(){
        return __( 'F j, Y g:i a', 'wp-admin-audit' );
    }

    protected static function getDefaultDateFormat(){
        return __( 'F j, Y', 'wp-admin-audit' );
    }
}